﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Media3D;
using MindFusion.Charting.Wpf;
using System.Windows.Media.Effects;

namespace SurfaceInteraction
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();

           
            surfaceChart.BeginInit();

            surfaceChart.PointMergeThreshold = 0;
           
            surfaceChart.YAxisSettings.MinValue = -1;
            surfaceChart.YAxisSettings.MaxValue = 1;
            surfaceChart.YAxisSettings.Interval = 0.2;

            surfaceChart.XAxisSettings.MinValue = 0;
            surfaceChart.XAxisSettings.MaxValue = 1;
            surfaceChart.XAxisSettings.Interval = 0.2;

            surfaceChart.ZAxisSettings.MinValue = 0;
            surfaceChart.ZAxisSettings.MaxValue = 1;
            surfaceChart.ZAxisSettings.Interval = 0.2;

            surfaceChart.XAxisSettings.LabelFontSize = 11;
            surfaceChart.XAxisSettings.LabelOffset = 3;
            surfaceChart.XAxisSettings.LabelFormat = "0.0";
            surfaceChart.XAxisSettings.TitleFontSize = 12;
            surfaceChart.XAxisSettings.TitleOffset = 5;
            surfaceChart.XAxisSettings.Title = "x/L(x)";

            surfaceChart.YAxisSettings.LabelFontSize = 11;
            surfaceChart.YAxisSettings.LabelOffset = 3;
            surfaceChart.YAxisSettings.LabelFormat = "0.0";
            surfaceChart.YAxisSettings.TitleFontSize = 12;
            surfaceChart.YAxisSettings.TitleOffset = 5;
            surfaceChart.YAxisSettings.TitleRotationAngle = 270;
            surfaceChart.YAxisSettings.Title = "Wave Function Amplitude";

            surfaceChart.ZAxisSettings.LabelFontSize = 11;
            surfaceChart.ZAxisSettings.LabelOffset = 3;
            surfaceChart.ZAxisSettings.LabelFormat = "0.0";
            surfaceChart.ZAxisSettings.TitleFontSize = 12;
            surfaceChart.ZAxisSettings.TitleOffset = 5;
            surfaceChart.ZAxisSettings.Title = "y/L(y)";


            surfaceChart.SideWallThickness = 0.5;
            surfaceChart.TextureType = TextureType.GradientColorMap;

            SortedList<double, Color> colorMapList = new SortedList<double, Color>();
            colorMapList.Add(-1.0, Color.FromArgb(255, 255, 0, 255));
            colorMapList.Add(-0.8, Color.FromArgb(255, 128, 0, 255));
            colorMapList.Add(-0.6, Color.FromArgb(255, 0, 0, 255));
            colorMapList.Add(-0.4, Color.FromArgb(255, 0, 127, 255));
            colorMapList.Add(-0.2, Color.FromArgb(255, 0, 255, 255));
            colorMapList.Add(0.0, Color.FromArgb(255, 0, 255, 0));
            colorMapList.Add(0.2, Color.FromArgb(255, 127, 255, 0));
            colorMapList.Add(0.4, Color.FromArgb(255, 255, 255, 0));
            colorMapList.Add(0.6, Color.FromArgb(255, 255, 127, 0));          
            colorMapList.Add(0.8, Color.FromArgb(255, 255, 0, 0));
           

            surfaceChart.ColorMapList = colorMapList;
            surfaceChart.MeshSelectionBrush = new SolidColorBrush(Color.FromArgb(150, 135, 206, 250));
            DirectionalLight light = new DirectionalLight()
            {
                Color = Colors.White,
                Direction = new Vector3D(-1, 3, -1),
            };
            surfaceChart.StaticLights.Add(light);
            surfaceChart.MouseDown += new MouseButtonEventHandler(surfaceChart_MouseDown);

            

          
            SeriesLegend legend = new SeriesLegend()
            {
                Margin = new Thickness(15, 0, 0, 0),
                Effect = new DropShadowEffect() { Opacity = 0.7, ShadowDepth = 3 },
            };

            Grid grid = new Grid();
            ClipToBounds = false;
            grid.Height = 20 * colorMapList.Count + 20;
            grid.ColumnDefinitions.Add(new ColumnDefinition() { Width = GridLength.Auto });
            grid.ColumnDefinitions.Add(new ColumnDefinition());

            double d = colorMapList.Count - 1;

            for (int i = colorMapList.Count - 1; i >= 0; i--)
            {
                Color color = colorMapList.Values[i];
                Color mid = Color.FromArgb(120, 255, 255, 255);
                LinearGradientBrush brush = new LinearGradientBrush();
                brush.StartPoint = new Point(0, 0);
                brush.EndPoint = new Point(1, 0);
                brush.GradientStops.Add(new GradientStop(Colors.Transparent, 0));
                brush.GradientStops.Add(new GradientStop(mid, 0.5));
                brush.GradientStops.Add(new GradientStop(Colors.Transparent, 1));

                Border border = new Border()
                {
                    Margin = new Thickness(5, (d - i) * 20, 5, 20),
                    Background = new SolidColorBrush(color),
                    Width = 20,
                };
                
                grid.Children.Add(border);



                Border border2 = new Border()
                {
                    Margin = new Thickness(5, (d - i) * 20, 5, 20),
                    Background = brush,
                    Width = 20,                   
                };
              
                grid.Children.Add(border2);



                TextBlock textBlock = new TextBlock()
                {
                    Text = colorMapList.Keys[i].ToString("0.0"),
                    Margin = new Thickness(5, (d - i) * 20 + 10, 5, 0),
                    VerticalAlignment = VerticalAlignment.Top,
                };
                Grid.SetColumn(textBlock, 1);              
                grid.Children.Add(textBlock);
            }

            TextBlock textBlock2 = new TextBlock()
            {
                Text = "1.0",
                Margin = new Thickness(5, -10, 5, 0),
                VerticalAlignment = VerticalAlignment.Top,
            };
            Grid.SetColumn(textBlock2, 1);
            grid.Children.Add(textBlock2);

            legend.Items.Add(grid);
            surfaceChart.Legends.Add(legend);


            RandomData();

            surfaceChart.EndInit();
        }

        void surfaceChart_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (hitCheck.IsChecked == true)
            {
                ChartHitTestResult3D result = surfaceChart.HitTest(e.GetPosition(this));
                Point3D realPoint = result.MeshPoint;
                Point3D dataPoint = result.DataPoint;
                Point3D? peak = result.ClosestPeak;

                if (peak == null)
                    return;

                string sx = realPoint.X.ToString("0.##");
                string sy = realPoint.Y.ToString("0.##");
                string sz = realPoint.Z.ToString("0.##");
                string realPointString = "- Mesh Point: X=" + sx + ", Y=" + sy + ", Z=" + sz;

                sx = dataPoint.X.ToString("0.##");
                sy = dataPoint.Y.ToString("0.##");
                sz = dataPoint.Z.ToString("0.##");
                string dataPointString = "- Data Point: X=" + sx + ", Y=" + sy + ", Z=" + sz;

                sx = peak.Value.X.ToString("0.##");
                sy = peak.Value.Y.ToString("0.##");
                sz = peak.Value.Z.ToString("0.##");
                string peakString = "- Closest Peak: X=" + sx + ", Y=" + sy + ", Z=" + sz;

                string seriesIndex = "- Series Index:" + surfaceChart.Series.IndexOf(result.Series).ToString();


                string title = "HitTest Result:";
                listView1.Items.Add(title);
                listView1.Items.Add(realPointString);
                listView1.Items.Add(dataPointString);
                listView1.Items.Add(peakString);
                listView1.Items.Add(seriesIndex);
            }
        }

        private void RandomData()
        {
            Random r = new Random();
            DoubleCollection dx = new DoubleCollection();
            DoubleCollection dy = new DoubleCollection();
            DoubleCollection dz = new DoubleCollection();
            for (int i = 0; i < 10; i++)
            {
                dx.Add(r.NextDouble() / 2);
                dy.Add(r.NextDouble() / 2 + 0.5);
                dz.Add(r.NextDouble());
            }
            for (int i = 0; i < 10; i++)
            {
                dx.Add(r.NextDouble() / 2 + 0.5);
                dy.Add(-(r.NextDouble() / 2 + 0.5));
                dz.Add(r.NextDouble());
            }


            foreach (SurfaceSeries s in surfaceChart.Series)
            {
                s.XData.Clear();
                s.YData.Clear();
                s.ZData.Clear();

                s.XData = dx.Clone();
                s.YData = dy.Clone();
                s.ZData = dz.Clone();
            }          
        }

        private void behavior_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (surfaceChart == null)
                return;

            ComboBox box = sender as ComboBox;
            if (box.Name == "behavior")
            {
                surfaceChart.Behavior = box.SelectedIndex == 0 ?
                    Behavior3D.ChangePerspective : Behavior3D.ChangeValue;
            }
            if (box.Name == "textureType")
            {
                surfaceChart.TextureType = box.SelectedIndex == 0 ?
                    TextureType.GradientColorMap : TextureType.ExactColorMap;
            }
            if (box.Name == "effect3D")
            {
                surfaceChart.Effect3D = box.SelectedIndex == 0 ?
                    ChartEffect3D.None : ChartEffect3D.ShaderEffect;
            }
        }

        private void meshClipHeight_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (surfaceChart == null)
                return;

            Slider slider = sender as Slider;
            if (slider.Name == "meshClipHeight")
            {
                if (surfaceChart.Series.Count > 0)
                {
                    (surfaceChart.Series[0] as SurfaceSeries).ClipMinimum = -15;
                    (surfaceChart.Series[0] as SurfaceSeries).ClipMaximum = e.NewValue;
                }
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            listView1.Items.Clear();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            SurfaceSeries series = new SurfaceSeries();
            surfaceChart.Series.Add(series);

            UpdateGroundLevel();
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            if (surfaceChart.Series.Count > 0)
                surfaceChart.Series.Remove(surfaceChart.Series[surfaceChart.Series.Count - 1]);

            UpdateGroundLevel();
        }

        private void UpdateGroundLevel()
        {
            double space = 2.0 / surfaceChart.Series.Count;
            double coef = ((surfaceChart.Series.Count - 1) * space) / 2.0;

            for (int i = 0; i < surfaceChart.Series.Count; i++)
            {
                SurfaceSeries serie = surfaceChart.Series[i] as SurfaceSeries;              
                serie.GroundLevelMargin = space * i - coef;
            }          
        }

        private void enableMeshSelection_Checked(object sender, RoutedEventArgs e)
        {
            if ((sender as CheckBox).IsChecked == true)
                surfaceChart.AllowMeshSelection = true;
            else
                surfaceChart.AllowMeshSelection = false;
        }       
    }
}
